'use strict';

var _classCallCheck = require('@babel/runtime/helpers/classCallCheck');
var _createClass = require('@babel/runtime/helpers/createClass');
var _defineProperty = require('@babel/runtime/helpers/defineProperty');
var utils = require('@forensic-js/utils');
var Wrapper = require('./Wrapper');
var nodeUtils = require('@forensic-js/node-utils');

var Router =
/*#__PURE__*/
function () {
  /**
   *
   * @param inheritMiddlewares - boolean indicating if parent middlewares should be inherited.
   */
  function Router(inheritMiddlewares) {
    _classCallCheck(this, Router);

    _defineProperty(this, "basePath", '');

    _defineProperty(this, "routes", {
      options: [],
      head: [],
      get: [],
      post: [],
      put: [],
      delete: [],
      all: []
    });

    _defineProperty(this, "middlewares", []);

    _defineProperty(this, "inheritMiddlewares", void 0);

    this.inheritMiddlewares = inheritMiddlewares;
  }
  /**
   * resolves the url route by joining it to the base path
   */


  _createClass(Router, [{
    key: "resolveUrl",
    value: function resolveUrl(url) {
      url = nodeUtils.joinPaths(this.basePath, utils.stripSlashes(url));
      return url !== '.' ? url : '';
    }
    /**
     * sets the route rule for a given http method
     *
     * @param method - the http method
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "set",
    value: function set(method, url, callback, options) {
      var resolvedOptions = utils.isNull(options) ? null : {
        middleware: utils.isObject(options) ? utils.makeArray(options.middleware) : utils.makeArray(options)
      };
      this.routes[method].push([this.resolveUrl(url), callback, resolvedOptions]);
      return this;
    }
    /**
     * returns the routes.
     */

  }, {
    key: "getRoutes",
    value: function getRoutes() {
      return this.routes;
    }
    /**
     * returns the middlewares
     */

  }, {
    key: "getMiddlewares",
    value: function getMiddlewares() {
      return this.middlewares;
    }
    /**
     * returns routing base path that gets prepended to all route and middleware urls
    */

  }, {
    key: "getBasePath",
    value: function getBasePath() {
      return this.basePath;
    }
    /**
     * sets routing base path that gets prepended to all route and middleware urls
    */

  }, {
    key: "setBasePath",
    value: function setBasePath(basePath) {
      this.basePath = utils.stripSlashes(basePath);
      return this;
    }
    /**
     * stores route rules for http OPTIONS method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "options",
    value: function options(url, callback) {
      var _options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      return this.set('options', url, callback, _options);
    }
    /**
     * stores route rules for http HEAD method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "head",
    value: function head(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('head', url, callback, options);
    }
    /**
     * stores route rules for http GET method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "get",
    value: function get(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('get', url, callback, options);
    }
    /**
     * stores route rules for http POST method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "post",
    value: function post(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('post', url, callback, options);
    }
    /**
     * stores route rules for http PUT method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "put",
    value: function put(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('put', url, callback, options);
    }
    /**
     * stores route rules for http DELETE method
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "delete",
    value: function _delete(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('delete', url, callback, options);
    }
    /**
     * stores route rules for all http methods
     *
     * @param url - route url
     * @param callback - route callback handler
     * @param options - route configuration object or middleware or array of middlewares
     */

  }, {
    key: "all",
    value: function all(url, callback) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      return this.set('all', url, callback, options);
    }
    /**
     * creates and returns a route wrapper for the given url
    */

  }, {
    key: "route",
    value: function route(url) {
      return new Wrapper(this, url);
    }
    /**
     * registers a middleware to be called whenever the given url is visited
     *
     * @param url - url to apply middleware to. use * to appy to all urls
     * @param middleware - the middleware or array of middlewares
     * @param options - middleware configuration option. here, you can specify the http method
     * that the middleware will run against
     */

  }, {
    key: "use",
    value: function use(url, middleware) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
      middleware = utils.makeArray(middleware);
      var resolvedOptions = utils.isNull(options) ? null : {
        method: utils.isObject(options) ? utils.makeArray(options.method) : utils.makeArray(options)
      };
      this.middlewares.push([this.resolveUrl(url), middleware, resolvedOptions]);
      return this;
    }
    /**
     * returns a boolean indicating if router inherits parent's middlewares when mounted
     * @param status - if given, it sets this value
     */

  }, {
    key: "shouldInheritMiddlewares",
    value: function shouldInheritMiddlewares(status) {
      if (utils.isBoolean(status)) {
        this.inheritMiddlewares = status;
      }

      return this.inheritMiddlewares;
    }
  }]);

  return Router;
}();

module.exports = Router;
//# sourceMappingURL=Router.js.map
