/// <reference types="node" />
import { ServerResponse } from 'http';
import FileServer from './FileServer';
import Request from './Request';
export default class Response extends ServerResponse {
    request: Request | null;
    fileServer: FileServer | null;
    startedAt: Date | null;
    endedAt: Date | null;
    constructor(req: Request);
    end(cb?: () => void): Promise<boolean>;
    end(data?: any, encoding?: string): Promise<boolean>;
    /**
     * sets response header
     * @param name response header name
     * @param value response header value
     */
    setHeader(name: string, value: string | number | string[]): this;
    /**
     * sets multiple response headers
     * @param headers object containing response header name value pairs
     */
    setHeaders(headers: {
        [p: string]: string | number | string[];
    }): this;
    /**
     * removes a single set response header at a time. function is chainable
     * @param name response header to remove
     */
    removeHeader(name: string): this;
    /**
     * remove response headers that are already set. function is chainable
     * @param names - comma separated list of header names to remove
     */
    removeHeaders(...names: string[]): this;
    /**
     * sets response status code
     * @param code - the response code
     */
    status(code: number): this;
    /**
     * sends json response back to the client.
     * @param data - the json string or json object which will be stringified
     */
    json(data: object | string): Promise<boolean>;
    /**
     * Redirect client to the given url
     */
    redirect(path: string, status?: number): Promise<boolean>;
    /**
     * sends a file download attachment to the client
     * @param filePath - relative or absolute file path
     * @param filename - suggested file download name
     */
    download(filePath: string, filename?: string): Promise<boolean>;
}
